# frozen_string_literal: true

require 'oj'
require_relative '../cache/redis_cache.rb'

module EodFacade
  class Dividends < ::EodFacade::Base
    class << self
      def call(symbol, params = {})
        unless Cache::RedisCache.get(redis_key(symbol, params[:from], params[:to]))
          response = make_request(url_path(symbol), params.merge(fmt: 'json'))
          
          unless response.success?
            raise ArgumentError, "Error fetching dividend data for #{symbol}"
          end

          Cache::RedisCache.set(
            redis_key(symbol, params[:from], params[:to]),
            response.parsed_response.to_json
          )
        end

        Oj.load(Cache::RedisCache.get(redis_key(symbol, params[:from], params[:to])))
      end

      private

      def url_path(symbol)
        "/div/#{symbol}"
      end

      def redis_key(symbol, from, to)
        "#{symbol}_#{from || ''}_#{to || ''}_dividend"
      end
    end
  end
end
